import urllib2
import urllib
import re
from sgmllib import SGMLParser


url = "https://translate.google.com/translate_dict"
google_url = "https://ajax.googleapis.com/ajax/services/language/translate"
referer_url = "https://edu.kde.org/parley/"
referer = "Referer: "+referer_url
header_referer = ("Referer",referer_url)
param_lang_pair = ("langpair","en|fr")
param_version = ("v","1.0")

class myParser(SGMLParser):

  def reset(self):
    SGMLParser.reset(self)
    self.words = []
    self.mystack = []
    self.spanstack = []

  def unknown_starttag(self,tag,attrs):
        self.mystack.append(tag)
        print "unknown : ", tag, " ", len(self.mystack)
  
  def start_span(self, attrs):
    found = False
    for name, value in attrs:
        if name == "class" and value == "definition":
            self.mystack.append("<translation>")
            found = True
    if not found:
      self.mystack.append("span")
  
  def report_unbalanced(self,tag):
   print "unbalanced : ",tag
   return

  def handle_data(self,data):
    if self.mystack[len(self.mystack)-1] == "<translation>":
        print "data: ", data
        self.words.append(data)
  
  def unknown_endtag(self,tag):
    #make it remove if self.mystack[len(self.mystack)-1] != tag
    if len(self.mystack) > 0:
        if self.mystack[len(self.mystack)-1] == tag:
            print "end tag: ", self.mystack[len(self.mystack)-1]
            self.mystack.pop()

def parserTest(data):
  p = myParser()
  p.feed(data)
  print p.words
  p.close()


def googleTranslation(word):
  param_word_trn = ("q",word)
  request_url = google_url + "?" + urllib.urlencode([param_version,param_word_trn,param_lang_pair])
  print request_url
  headers = urllib.urlencode([header_referer])
  results = urllib2.urlopen(request_url,headers)
  data = results.read()
  
def googleTranslation2(word):
  param_word_trn = ("q",word)
  request_url = google_url + "?" + urllib.urlencode([param_version,param_word_trn,param_lang_pair])
  print request_url
  r = urllib2.Request(request_url)
  r.add_header("Referer",referer_url)
  results = urllib2.urlopen(r)
  data = results.read()
  return data

def fetchTranslation(word):
  param_word_trn = ("q",word)
  request_url = url + "?" + urllib.urlencode([param_word_trn,param_lang_pair])
  print request_url
  results = urllib2.urlopen(request_url)
  data = results.read()
  #p = re.compile('<span\ class="definition">(?:<span class="comment">.*</span>)*([^<>]*)<span class="comment">',re.IGNORECASE)
  p = re.compile('<span\ class="definition">([^<>]*)<span class="comment">',re.IGNORECASE)
  l = p.findall(data)
  parserTest(data)
  return l

def returnlist():
    L = [1,2,3,4,5]
    return L

def testTuple():
  return [("first","second"),("first1","second1")]

print fetchTranslation("love")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import logging

from oauthlib.common import extract_params
from oauthlib.oauth1 import Client, SIGNATURE_HMAC, SIGNATURE_TYPE_AUTH_HEADER
from oauthlib.oauth1 import SIGNATURE_TYPE_BODY
from requests.compat import is_py3
from requests.utils import to_native_string
from requests.auth import AuthBase

CONTENT_TYPE_FORM_URLENCODED = 'application/x-www-form-urlencoded'
CONTENT_TYPE_MULTI_PART = 'multipart/form-data'

if is_py3:
    unicode = str

log = logging.getLogger(__name__)

# OBS!: Correct signing of requests are conditional on invoking OAuth1
# as the last step of preparing a request, or at least having the
# content-type set properly.
class OAuth1(AuthBase):
    """Signs the request using OAuth 1 (RFC5849)"""

    client_class = Client

    def __init__(self, client_key,
            client_secret=None,
            resource_owner_key=None,
            resource_owner_secret=None,
            callback_uri=None,
            signature_method=SIGNATURE_HMAC,
            signature_type=SIGNATURE_TYPE_AUTH_HEADER,
            rsa_key=None, verifier=None,
            decoding='utf-8',
            client_class=None,
            force_include_body=False,
            **kwargs):

        try:
            signature_type = signature_type.upper()
        except AttributeError:
            pass

        client_class = client_class or self.client_class

        self.force_include_body = force_include_body

        self.client = client_class(client_key, client_secret, resource_owner_key,
            resource_owner_secret, callback_uri, signature_method,
            signature_type, rsa_key, verifier, decoding=decoding, **kwargs)

    def __call__(self, r):
        """Add OAuth parameters to the request.

        Parameters may be included from the body if the content-type is
        urlencoded, if no content type is set a guess is made.
        """
        # Overwriting url is safe here as request will not modify it past
        # this point.
        log.debug('Signing request %s using client %s', r, self.client)

        content_type = r.headers.get('Content-Type', '')
        if (not content_type and extract_params(r.body)
                or self.client.signature_type == SIGNATURE_TYPE_BODY):
            content_type = CONTENT_TYPE_FORM_URLENCODED
        if not isinstance(content_type, unicode):
            content_type = content_type.decode('utf-8')

        is_form_encoded = (CONTENT_TYPE_FORM_URLENCODED in content_type)

        log.debug('Including body in call to sign: %s',
                  is_form_encoded or self.force_include_body)

        if is_form_encoded:
            r.headers['Content-Type'] = CONTENT_TYPE_FORM_URLENCODED
            r.url, headers, r.body = self.client.sign(
                unicode(r.url), unicode(r.method), r.body or '', r.headers)
        elif self.force_include_body:
            # To allow custom clients to work on non form encoded bodies.
            r.url, headers, r.body = self.client.sign(
                unicode(r.url), unicode(r.method), r.body or '', r.headers)
        else:
            # Omit body data in the signing of non form-encoded requests
            r.url, headers, _ = self.client.sign(
                unicode(r.url), unicode(r.method), None, r.headers)

        r.prepare_headers(headers)
        r.url = to_native_string(r.url)
        log.debug('Updated url: %s', r.url)
        log.debug('Updated headers: %s', headers)
        log.debug('Updated body: %r', r.body)
        return r



#!/usr/bin/env kf5kross
import Parley
import Kross
import PyKDE4
from PyQt4 import QtGui
from PyQt4 import QtCore
from PyQt4 import uic
import sys


def testPyQt():
  print "Testing pyQt"
  #app = QtGui.QApplication(sys.argv)
  #(formClass, baseClassu) = uic.loadUiType(Parley.pluginDirs()[0]+"./mywidget.ui")

  #widget = QtGui.QDialog()
  #widget.resize(250, 150)
  #widget.setWindowTitle('simple')
  #c = formClass()
  #print c
  #c.setupUi(widget)
  #widget.exec_()
  #app.exec_()
  
def testPyKDE():
  dialog = PyKDE4.kdeui.KDialog()


def testUI():
 forms = Kross.module("forms")
 dialog = forms.createDialog("My Dialog")
 dialog.setButtons("Ok|Cancel")
 print dir(dialog)
 page = dialog.addPage("Picture Selector","Select an image","document-open")
 #l = forms.createLayout(dialog,"QVBoxLayout")
 #label = forms.createWidget(page,"QLabel")
 #label.text = "Hello World Label"
 btn = forms.createWidget(page,"QPushButton")
 btn.text = "Next Picture"
 gview = forms.createWidget(page,"QGraphicsView")
 print dir(gview)
 if dialog.exec_loop():
     forms.showMessageBox("Information", "Okay...", "The Ok-button was pressed")


def click():
  print "click"

def testUI2():
  #print Parley.dataDirs()
  #print Parley.pluginDirs()
  #return
  forms = Kross.module("forms")
  mydialog = forms.createDialog("MyDialog")
  mydialog.setButtons("Ok|Cancel")
  mydialog.setFaceType("Plain") #Auto Plain List Tree Tabbed
  
  print dir(mydialog)
  #for attr in dir(mydialog):
    #print attr, getattr(mydialog,attr)
  mywidget = forms.createWidgetFromUIFile(mydialog, Parley.pluginDirs()[0]+"./mywidget.ui")
  mydialog.setMainWidget(mywidget)
  mywidget["QTextEdit"].setText("some string")
  #print dir(mywidget["QPushButton"])
  mywidget["QPushButton"].text = "Push me"
  #mywidget.textEdit.setText("soooome string")
  Parley.connect(mywidget["QPushButton"],"pressed()",click)
  #print mywidget
  print dir(mywidget)
  if mydialog.exec_loop():
      if mydialog.result() == "Ok":
          print mywidget["QTextEdit"].text


def init():
  print "init: myscript1"

def translateWord(word,fromLang,toLang):
  print "myscript1: Translating Word ...", word, fromLang, toLang
  Parley.addTranslation("Hello","en_US","fr","bonbon")
  
def addNewEntry():
  print "Adding new entry"
  newEntry = Parley.activeLesson.newEntry()
  newEntry.translation(0).text = "boy"
  newEntry.translation(1).text = "hijo"
  Parley.activeLesson.appendEntry(newEntry)
  
def addNewLesson():
  newlesson = Parley.document.newLesson("Hello There!!");
  lesson.appendChildLesson(newlesson)
  newEntry = newlesson.newEntry()
  newEntry.translation(0).text = "Hello"
  newEntry.translation(1).text = "Hola"
  newlesson.appendEntry(newEntry)
  
def setWordType():
  for entry in Parley.doc.rootLesson.entries(True):
    for tr in entry.translations():
        Parley.doc.setWordType(tr,"Noun")
        print tr.wordType()
        
def addNewTenses():
  for ident in Parley.doc.identifiers():
    T = ident.tenses()
    T.append("Present Perfect")
    T.append("Past Simple")
    T.append("Past Perfect")
    ident.setTenses(T)
    print ident.tenses()
  
def conjugations():
  Parley.doc.setTenses(["present","past"])
  print Parley.doc.tenses()
  for entry in Parley.doc.rootLesson.entries(True):
    for tr in entry.translations():
        if tr.wordType() == "Verb":
          print "setting conjugation"
          print tr.text
          #tr.setConjugation("first","present","Plural","First")
          tr.setConjugation("first","present",Parley.Plural,Parley.First)
          tr.setConjugation("third","present",Parley.Plural,Parley.Third)
          tr.setConjugation("male","present",Parley.Plural,Parley.ThirdMale)
          #tr.setConjugation("female","present","Plural","ThirdFemale")
          #tr.setConjugation("playing","past","plural","Third")
          #tr.setConjugation("se","present","Plural","Second")
          #tr.setConjugation("dualthird","present","Dual","Third")
          #print tr.conjugation("past","plural","Third")
          print tr.conjugation("present")
          print tr.conjugation("present",Parley.Plural)
  
def testKUrl():
  l = Parley.activeLesson
  print l.name
  print l.imageUrl
  l.imageUrl = "/home/kde-devel/"
  print l.imageUrl
  #print l.imageUrl()
  #l.setImageUrl("/home/kde-devel/image.jpg");
  #print l.imageUrl()
  
def registerActions():
  newaction = Parley.newAction("my_script1")
  print newaction
  newaction.text="My script 1"
  Parley.connect(newaction,"triggered()",actionFunction)
  
  newaction2 = Parley.newAction("my_script1","My script 2a")
  Parley.connect(newaction2,"triggered()",actionFunction2)
  
  newaction3 = Parley.newAction("my_script1","ui")
  Parley.connect(newaction3,"triggered()",testUI)
  
  newaction4 = Parley.newAction("my_script1","ui2")
  Parley.connect(newaction4,"triggered()",testUI2)
  
  newaction5 = Parley.newAction("my_script1","pyQt")
  Parley.connect(newaction5,"triggered()",testPyQt)
  
  newaction6 = Parley.newAction("my_script1","pyKDE")
  Parley.connect(newaction6,"triggered()",testPyKDE)

  return

def appendChildrenToAllLessons():
  children = Parley.activeLesson.childLessons()
  print children
  i = 1
  for child in children:
    print dir(child)
    newlesson = Parley.document.newLesson("Testa!! " + str(i))
    child.appendChildLesson(newlesson)
    i = i + 1
  return
  
def testcode():
 new_entry = Parley.activeLesson.newEntry()
 new_entry.setTranslations(["good morning","bonjour"])
 print new_entry.active
 new_entry.active = False
 Parley.activeLesson.appendEntry(new_entry)
 return
 #Parley.doc.setTenseName(0,"present")
 #Parley.doc.setTenseName(1,"past simple")
 for entry in Parley.doc.rootLesson.entries(True):
    for tr in entry.translations():
        print tr.wordType()
        if tr.text == "play":
            Parley.doc.setWordType(tr,"Verb")
            tr.setConjugation("plays","present","singular","third")
            tr.setConjugation("played","past simple","singular","first")
 return
 for entry in Parley.document.rootLesson.entries(True):
    for translation in entry.translations():
      print translation.text
      print translation.practiceDate()
      print translation.practiceDate("dd/MM/yyyy")


def newDocument():
  doc = Parley.newDocument()
  doc.title = "New document"
  #set identifiers
  doc.appendNewIdentifier("English","en_US")
  doc.appendNewIdentifier("French","fr")
  #lessons
  l1 = doc.newLesson("Lesson1")
  doc.rootLesson.appendChildLesson(l1)
  #first way
  e = l1.newEntry()
  e.setTranslation(0,"dog")
  e.setTranslation(1,"chien")
  l1.appendEntry(e)
  #second way
  ee = l1.newEntry(["glass","verre"])
  l1.appendEntry(ee)
  #third way
  ee = l1.appendNewEntry(["book","livre"])
  
  #new lesson (fast way)
  l2 = doc.appendNewLesson("Lesson 2")
  l2.appendNewEntry(["I","je"]);
  
  #new lesson under Lesson 2
  l3 = doc.appendNewLesson("Lesson 3",l2)
  l3.appendNewEntry(["good morning","bonjour"])
  
  l5 = doc.newLesson("Lesson 5")
  doc.appendLesson(l5)
  
  #fl = doc.findLesson("Lesson 3")
  #if fl != None:
    #print "found"
    #print fl.name
  #else:
    #print "not found"
  
  #save document
  doc.saveAs("/home/kde-devel/test_new_document.kvtml")
  
def tryArticle():
   #add new one
  newid = Parley.doc.newIdentifier()
  newid.name = "English"
  newid.locale = "en_US"
  newid.setArticle("o",Parley.Singular,Parley.Definite,Parley.Masculine)
  newid.setArticle("h",Parley.Singular,Parley.Definite,Parley.Feminine)
  newid.setArticle("to",Parley.Singular,Parley.Definite,Parley.Neutral)
  newid.setPersonalPronoun("oi",Parley.Plural,Parley.First)
  print newid.personalPronoun(Parley.Plural,Parley.First)
  print newid.article(Parley.Singular,Parley.Definite,Parley.Neutral)
  index = Parley.doc.appendIdentifier(newid)
  
def tryArticle2():
   #add new one
  newid = Parley.doc.newIdentifier()
  newid.name = "English"
  newid.locale = "en_US"
  newid.setArticle("o",Parley.Singular|Parley.Definite|Parley.Masculine)
  newid.setArticle("h",Parley.Singular|Parley.Definite|Parley.Feminine)
  newid.setArticle("to",Parley.Singular|Parley.Definite|Parley.Neuter)
  newid.setArticle("ta",Parley.Singular|Parley.Indefinite|Parley.Masculine)
  #newid.setPersonalPronoun("oi",Parley.Plural,Parley.First)
  #print newid.personalPronoun(Parley.Plural,Parley.First)
  print newid.article(Parley.Singular|Parley.Definite|Parley.Neuter)
  index = Parley.doc.appendIdentifier(newid)
  
def GermanArticles():
    newid = Parley.doc.newIdentifier()
    newid.name = "German"
    newid.locale = "fr"
    newid.setArticle("der",Parley.Singular,Parley.Definite,Parley.Masculine)
    newid.setArticle("die",Parley.Singular,Parley.Definite,Parley.Feminine)
    newid.setArticle("das",Parley.Singular,Parley.Definite,Parley.Neutral)
    newid.setArticle("ein",Parley.Singular,Parley.Indefinite,Parley.Masculine)
    newid.setArticle("eine",Parley.Singular,Parley.Indefinite,Parley.Feminine)
    newid.setArticle("ein",Parley.Singular,Parley.Indefinite,Parley.Neutral)
    newid.setPersonalPronoun("ich",Parley.Singular,Parley.First)
    newid.setPersonalPronoun("du",Parley.Singular,Parley.Second)
    newid.setPersonalPronoun("er",Parley.Singular,Parley.ThirdMale)
    newid.setPersonalPronoun("sie",Parley.Singular,Parley.ThirdFemale)
    newid.setPersonalPronoun("es",Parley.Singular,Parley.Third)
    newid.setPersonalPronoun("wir",Parley.Plural,Parley.First)
    newid.setPersonalPronoun("ihr",Parley.Plural,Parley.Second)
    newid.setPersonalPronoun("sie",Parley.Plural,Parley.Third)
    print newid.personalPronouns()
    index = Parley.doc.appendIdentifier(newid)
    
    #how to specify the articles and personal pronouns of a language (German in this example)
def GermanArticlesNew():
    newid = Parley.doc.newIdentifier()
    newid.name = "German"
    newid.locale = "de"
    newid.setArticle("der",Parley.Singular|Parley.Definite|Parley.Masculine)
    newid.setArticle("die",Parley.Singular|Parley.Definite|Parley.Feminine)
    newid.setArticle("das",Parley.Singular|Parley.Definite|Parley.Neuter)
    newid.setArticle("ein",Parley.Singular|Parley.Indefinite|Parley.Masculine)
    newid.setArticle("eine",Parley.Singular|Parley.Indefinite|Parley.Feminine)
    newid.setArticle("ein",Parley.Singular|Parley.Indefinite|Parley.Neuter)
    newid.setPersonalPronoun("ich",Parley.Singular|Parley.First)
    newid.setPersonalPronoun("du",Parley.Singular|Parley.Second)
    newid.setPersonalPronoun("er",Parley.Singular|Parley.Third|Parley.Masculine)
    newid.setPersonalPronoun("sie",Parley.Singular|Parley.Third|Parley.Feminine)
    newid.setPersonalPronoun("es",Parley.Singular|Parley.Third|Parley.Neuter)
    newid.setPersonalPronoun("wir",Parley.Plural|Parley.First)
    newid.setPersonalPronoun("ihr",Parley.Plural|Parley.Second)
    newid.setPersonalPronoun("sie",Parley.Plural|Parley.Third)
    #print newid.personalPronouns()
    index = Parley.doc.appendIdentifier(newid)
  
def testEnums():
  for entry in Parley.doc.rootLesson.entries(True):
    tr = entry.translation(0)
    print tr
    tr.setDeclension("Paizontas",Parley.Singular,Parley.Genitive)
    tr.setDeclension("Paizomenos",Parley.Singular,Parley.Accusative)
    print tr.declension(Parley.Singular,Parley.Genitive).text
    print tr.declension(Parley.Singular,Parley.Accusative).text
    return
  
def testSetEntries():
  l = Parley.activeLesson
  entries = []
  #entries.append(l.newEntry(["good morning","bonjoura"]))
  entries.append(l.newEntry(["ela","paenne"]))
  print entries
  l.setEntries(entries)
  
def actionFunction():
  print "Action called!!"
  #addNewTenses()
  GermanArticlesNew()
  #for entry in Parley.doc.rootLesson.entries(True):
    #print entry.translationTexts()
  #testSetEntries()
  #for entry in Parley.selectedEntries():
    #print entry.translationTexts()
  #testEnums()
  #tryArticle2()
  #testUI()
  #GermanArticles()
  #newDocument()
  #conjugations()
  #print Parley.doc.wordTypes()
  #setWordType()
  #testcode()
  #appendChildrenToAllLessons()
  
  #for lesson in Parley.document.allLessons():
    #print lesson.name
  
  #children = Parley.activeLesson.childLessons(True)
  #for child in children:
    #print child.name
    

def actionFunction_old():
  print "Action called!!"
  entries = Parley.activeLesson.entries(True)
  
  for entry in entries:
    s = ""
    for tr in entry.translations:
      print tr
      if tr.text:
        s += tr.text + " "
    print s;
    
  print "sublessons"
  for sublesson in Parley.activeLesson.childLessons:
    print sublesson.name

  return

def actionFunction2():
  print "Action2 called!!"
  tenses = Parley.doc.tenses()
  print tenses
  tenses.append("Present Perfect")
  tenses.append("Past Simple")
  Parley.doc.setTenses(tenses)
  print Parley.doc.tenses()
  return


def test():
    print "Test from myscript1.py"
    testAction()
  
def test_old():
  print "Test from myscript1.py"
  print Parley.document.rootLesson.name
  #addNewEntry()
  testKUrl()
  #Parley.open('/home/kde-devel/My\ Vocabularies/test2.kvtml')
  #print Parley.activeLesson.name
  #Parley.activeLesson.name = "Hi!!"
  lesson = Parley.activeLesson
  #print lesson.getEntries()
  
  print lesson.name
  print "Child lessons"
  print lesson.childLessonCount()
  for i in range(0,lesson.childLessonCount()):
    print " ",lesson.childLesson(i).name
    
  print lesson.entry(0,True).translationIndices
  print lesson.entry(0,True).translation(0).text
  print lesson.entry(0,True).translation(1).text
  print lesson.entry(0,True).translation(1).practiceDate
  print "Listing"
  print dir(lesson.entry(0,True).translation(1))
  
  print "Translation attributes"
  print len(dir(lesson.entry(0,True).translation(1)))
  for attr in dir(lesson.entry(0,True).translation(1)):
    a = getattr(lesson.entry(0,True).translation(1),attr)
    print attr, a

  for entry in lesson.entries(true):
     print entry.translation(0).text,lesson.entry(i,True).translation(1).grade
     
  for i in range(0,lesson.entryCount(True)):
    entry = lesson.entry(i,True)
    if entry.translation(0).text == "girl":
      entry.translation(0).text = "boy"

  print lesson.entryCount()
  lesson.removeEntry(0)
  print lesson.entryCount()

#print Parley.locales()
#print Parley.localeName("en_US")

#doc = Parley.document()
#doc.callFromScriptTest()

#Parley.document.callFromScriptTest()

#doc = Parley.document
#doc.name = "Hi"
#doc.printName()

def translateFromInternet(word,fromLang,toLang):
    print "Translating from Internet!!.."

Parley.connect("translateWord(const QString &,const QString &,const QString &)",translateFromInternet)

print "TEST ACTION"

registerActions()

#print "OPENING DOCUMENT"
##Parley.open('/home/kde-devel/My\ Vocabularies/test.kvtml')

#print "myscript1"
#init()
#Parley.callFromScript()
#Parley.connect("signalTranslateWord(QString)",translate)
#init()




#!/usr/bin/env kf5kross

import socket
import urllib2
import urllib
from sgmllib import SGMLParser
import Parley
import re

# timeout of search (important for slow connections, not to freeze Parley by waiting for a result)
timeout = 1.0
socket.setdefaulttimeout(timeout)

# fetches the html document for the given word and language pair
def fetchData(word,from_lang,to_lang):
  url = "https://translate.google.com/translate_dict"
  param_word_trn = ("q",word)       #set query parameter
  param_lang_pair = ("langpair",from_lang+"|"+to_lang)
  request_url = url + "?" + urllib.urlencode([param_word_trn,param_lang_pair])
  #print request_url
  try:
    results = urllib2.urlopen(request_url)
    return results.read()
  except:
    #in case of error not to return incompleted results
    return ""

#parses data and returns the parser object (that contains the translations/langpairs found)
def parseData(data,word,from_lang,to_lang):
  p = myParser()
  p.word = word
  p.from_lang = from_lang
  p.to_lang = to_lang
  p.feed(data)
  p.close()
  return p

#corrects the difference between the locale names of Parley and the google dictionary
def locale(lang):
  if lang == "en_US":
    return "en"
  if lang == "zh_TW":
    return "zh-TW"
  if lang == "zh_HK":
    return "zh-HK"
  if lang == "zh_CN":
    return "zh-CN"

  return lang

# called by Parley to translate the word
def translateWord(word,from_lang,to_lang):
  print "google_dictionary.py - Translating",word,from_lang,to_lang
  data = fetchData(word,locale(from_lang),locale(to_lang))
  parser = parseData(data,word,from_lang,to_lang)
  #return parser.words

# called by Parley to retrieve the language pairs provided by this script
# should return: [("en","fr"),("en","de")] for translation from english to french and english to german
def getLanguagePairs():
  data = fetchData("ignorethis","en","fr")
  parser = parseData(data)
  return map(split_langpair,parser.langpairs)

# function to split a language pair string into a tuple
def split_langpair(s):
  [f,t] = s.split("|",1)
  return (f,t)

# ------------ HTML Parser ----------- #

class myParser(SGMLParser):
  #for every start_tagname function you add you have to make sure the tag is added to the self.tags_stack

  def reset(self):
    SGMLParser.reset(self)
    self.words = []         #translated words found in html
    self.langpairs = []     #language pairs found in html file
    self.tags_stack = []
    self.stop = False

  def unknown_starttag(self,tag,attrs):
    self.tags_stack.append(tag)
    #print "unknown : ", tag, " ", len(self.tags_stack)

  def start_span(self, attrs):
    #print "known : ", "span", " ", len(self.tags_stack)
    if ("class","definition") in attrs:
        self.tags_stack.append("<!translation!>") #marks tag with <!translation!> to get its data in handle_data
    else:
        self.tags_stack.append("span")

  def start_option(self, attrs):
    for name,value in attrs:
      if name == "value":
        self.langpairs.append(value)
    self.tags_stack.append("option")

  def handle_data(self,data):
    if data == "Web definitions": self.stop = True #to make it stop after the web definitions
    if len(self.tags_stack) > 0 and self.tags_stack[len(self.tags_stack)-1] == "<!translation!>":
        #print "data: ", data
        self.words.append(data.strip())
        #print self.word, self.from_lang, self.to_lang
        if self.stop == False:
          w = self.clearWord(data)
          Parley.addTranslation(self.word,self.from_lang,self.to_lang,w)
  
  def unknown_endtag(self,tag):
    myParser.remove_not_closed_tags(self,tag)
    if len(self.tags_stack) > 0 and self.tags_stack[len(self.tags_stack)-1] == tag:
        #print "end_tag : ", tag, " ", len(self.tags_stack)
        self.tags_stack.pop()

  #removes all the tags from the stack that have no closed tags (don't modify)
  def remove_not_closed_tags(self,tag):
    while len(self.tags_stack) > 0 and self.tags_stack[len(self.tags_stack)-1] != tag:
      self.tags_stack.pop()

  #cleans up the given word from parentheses etc
  def clearWord(self,word):
    #word = "b[lue] socks (and) red shoes"
    p = re.compile( '(jmdn\.|etw\.)')
    word = p.sub( '', word)
    p = re.compile( '(\(.*\))')
    word = p.sub( '', word)
    p = re.compile( '(\[.*\])')
    word = p.sub( '', word)
    p = re.compile( '(\W)',re.UNICODE)
    word = p.sub( ' ', word)
    #replace double spaces produced from the previous ones
    p = re.compile( '(\s\s)')
    word = p.sub( ' ', word)
    return word.strip()



#!/usr/bin/env kross
# -*- coding: utf-8 -*-
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
# Copyright 2009 Frederik Gladhorn <gladhorn@kde.org>
# Copyright 2009 Thomas Eitzenberger <eth@gmx.at>
# Parts of this file are based on the google_images.py script by Avgoustinos Kadis

import Parley
import mwclient
import sys
import os
from os import path
import string
import Kross
import urllib
from urlparse import urlparse

T = Kross.module("kdetranslation")

# fetch http_content
def fetch_http_content(url, params, method):
    params = urllib.urlencode(params)
    if method=='POST':
        f = urllib.urlopen(url, params)
    else:
        f = urllib.urlopen(url+'?'+params)
    return (f.read(), f.code)

# Connected to the action menu
def fetchSound():
    print "fetching sound"
    tr = Parley.selectedTranslations()
    for word in tr:
        print "checking sound for " + word.text
        fetchSoundForTranslation(word)

def fetchSoundForTranslation(word):
    locale = getLocale(word)
    wikiLocales = getWikiLocales(locale)
    bFound = 0
    for wikiLocale in wikiLocales:
        if (bFound == 0):
            print 'Language:', wikiLocale, ' Word: ', word.text
            soundfile = getWikiObject(wikiLocale, word.text, None)
            # check for specific pronunciation for nouns
            if soundfile == None:
                soundfile = getWikiObject(wikiLocale, word.text, 'noun')
            # check for specific pronunciation for verbs
            if soundfile == None:
                soundfile = getWikiObject(wikiLocale, word.text, 'verb')
            if soundfile:
                url = downloadFromWiki(soundfile)
                word.soundUrl = url
                bFound = 1
                print "setting url for word " + word.text + ":" + word.soundUrl
            else:
                print "No translation found for: ", word.text

# locale of the given translation
def getLocale(trans):
    sel_entries = Parley.selectedEntries()
    for entry in sel_entries:
        for i in entry.translationIndices():
            if entry.translation(i).text == trans.text:
                return Parley.doc.identifier(i).locale

# go from iso codes to wiktionary
def getWikiLocales(lang):
    langs = []
    if (string.lower(lang[:2]) == 'en'):
        langs.append('En-us')
        langs.append('En-uk')
        langs.append('En-ca')
    elif (string.lower(lang[:2]) == 'de'):
        langs.append('De')
        langs.append('De-at')
    elif (string.lower(lang[:2]) == 'fr'):
        langs.append('Fr')
    else:
        langs.append(lang.capitalize())
    return langs

def getWikiObject(lang, word, appendix):
    commons = mwclient.Site('commons.wikimedia.org')
    # TODO: figure out if some utf-8 magic is needed here
    query = lang + '-' + word
    if (appendix != None):
        query += '-' + appendix

    query += '.ogg'
    soundfile = commons.Images[query]
    print 'Query: ', query, 'File: ', soundfile.name.encode('utf-8'), ' Exists: ', soundfile.exists
    if soundfile.exists:
        return soundfile
    return None

def downloadFromWiki(wikiObject):
    filesdir = getTargetDirectory()
    if not path.exists(filesdir):
        os.mkdir(filesdir)
    filename = filesdir + wikiObject.name.encode('utf-8')
    print filename
    saveas = open(filename,'w')
#   remote = wikiObject.download()
#   workaround: because download method results with http response code 404
    content, response_code = fetch_http_content(
                              wikiObject.imageinfo['url'],
                              {},
                              'GET'
                         )
    print 'HTTP Response code: ', response_code
    if(response_code == 200):
	    saveas.write(content)
    saveas.close()
    return filename

# directory the files are saved to (subdirectory of the document)
def getTargetDirectory():
    filepath = Parley.doc.url
    if filepath.startswith('file://'):
        url = urlparse(filepath)
        filepath = path.abspath(path.join(url.netloc, url.path))
    p,f = path.split(filepath)
    [name,ext] = f.rsplit(".", 1)
    filesdir = p + "/" + name + "_files/"
    return filesdir

#create a new action in Parley's script menu
action = Parley.newAction("fetch_sound",T.i18n("Fetch Sound"))
action.statusTip=T.i18n("Fetches a sound file from commons.wikimedia.org for the selected word")
Parley.connect(action,"triggered()",fetchSound)



#!/usr/bin/env kf5kross
# -*- coding: utf-8 -*-

import socket
import urllib2
import urllib
from sgmllib import SGMLParser
import Parley
import re

#german to french (Glas) [links off]
#https://dict.leo.org/frde?lp=frde&lang=de&searchLoc=1&cmpType=relaxed&sectHdr=on&spellToler=on&chinese=both&pinyin=diacritic&search=Glas&relink=off

#german to spanish (Glas)
#https://dict.leo.org/esde?lp=esde&lang=de&searchLoc=1&cmpType=relaxed&sectHdr=on&spellToler=std&chinese=both&pinyin=diacritic&search=Glas&relink=on

#spanish to german (tarro)
#https://dict.leo.org/esde?lp=esde&lang=de&searchLoc=-1&cmpType=relaxed&sectHdr=on&spellToler=std&chinese=both&pinyin=diacritic&search=tarro&relink=on

#spanish to german (tarro) [links off]
#https://dict.leo.org/esde?lp=esde&lang=de&searchLoc=-1&cmpType=relaxed&sectHdr=on&spellToler=std&chinese=both&pinyin=diacritic&search=tarro&relink=off

# timeout of search (important for slow connections, not to freeze Parley by waiting for a result)
timeout = 1.0
socket.setdefaulttimeout(timeout)

def languageString(from_lang,to_lang):
  combined = from_lang + "-" + to_lang
  A = { 'es-de': 'esde', 'de-es':'esde', 'fr-de':'frde', 'de-fr':'frde', 'en-de':'ende', 'de-en':'ende', 'it-de':'itde', 'de-it':'itde', 'ch-de':'chde', 'de-ch':'chde' }
  if (A.has_key(combined)):
    return A[combined]
  else:
    return None

def searchLoc(from_lang,to_lang):
  if from_lang == "de":
    return 1
  return -1

# fetches the html document for the given word and language pair
def fetchData(word,from_lang,to_lang):
  locale = languageString(from_lang,to_lang)
  if locale == None:
    return #not supported languages
  url = "https://dict.leo.org/"+locale
  #esde?lp=esde&lang=de&searchLoc=-1&cmpType=relaxed&sectHdr=on&spellToler=std&chinese=both&pinyin=diacritic&search=tarro&relink=off
  params = [("lp",locale),("lang","de"),("searchLoc",searchLoc(from_lang,to_lang)),("cmpType","relaxed"),("sectHdr","on"),("spellToler","std"),("chinese","both"),("pinyin","diacritic"),("search",word),("relink","off")]

  #param_word_trn = ("q",word)       #set query parameter
  #param_lang_pair = ("langpair",from_lang+"|"+to_lang)
  request_url = url + "?" + urllib.urlencode(params)
  #print request_url
  try:
    results = urllib2.urlopen(request_url)
    return results.read()
  except:
    #in case of error not to return incompleted results
    return ""

#parses data and returns the parser object (that contains the translations/langpairs found)
def parseData(data,word,from_lang,to_lang):
  p = myParser()
  p.word = word
  p.from_lang = from_lang
  p.to_lang = to_lang
  p.feed(data)
  p.close()
  return p

#corrects the difference between the locale names of Parley and the google dictionary
def locale(lang):
  if lang == "en_US":
    return "en"
  if lang == "en_GB":
    return "en"
  if lang == "zh_TW":
    return "zh-TW"
  if lang == "zh_HK":
    return "zh-HK"
  if lang == "zh_CN":
    return "zh-CN"

  return lang

# called by Parley to translate the word
def translateWord(word,from_lang,to_lang):
  print "dict-leo.py - Translating:",word, "from", from_lang, "to", to_lang
  data = fetchData(word,locale(from_lang),locale(to_lang))
  if data != None:
    print "Data Fetched for:",word
    #print data
  #print data
  parser = parseData(data,word,from_lang,to_lang)
  #return parser.words

# called by Parley to retrieve the language pairs provided by this script
# should return: [("en","fr"),("en","de")] for translation from english to french and english to german
def getLanguagePairs():
  data = fetchData("ignorethis","en","fr")
  parser = parseData(data)
  return map(split_langpair,parser.langpairs)

# function to split a language pair string into a tuple
def split_langpair(s):
  [f,t] = s.split("|",1)
  return (f,t)

# ------------ HTML Parser ----------- #

class myParser(SGMLParser):
  #for every start_tagname function you add you have to make sure the tag is added to the self.tags_stack

  def reset(self):
    SGMLParser.reset(self)
    #self.tags_stack = []
    self.td_data_stack = []
    self.in_results_table = False
    self.td_data = ""
    self.td_after = 0
    self.keyword_found = False
    self.in_translation_td = False
    self.in_small = False

  def start_table(self,attrs):
    #mark the results table
    if ("id","results") in attrs:
      self.in_results_table = True
    
  #to ignore the <small> tags
  def start_small(self,attrs):
    self.in_small = True
    
  #def end_small(self):
    #self.in_small = False
    
  #checks if the word contained by this <td> (self.td_data) matches the keyword. If yes then the corresponding td_data (previous/next two) is added as a translation
  def end_td(self):
    #print "end of a table data"
    #print "-" ,self.td_data , "-"
    #print myParser.clearWord(self,self.td_data)

    self.td_data = myParser.clearWord(self,self.td_data)

    #matching on the second column (german) and getting the translation from 2 column's on the left
    if self.td_data.lower() == self.word.lower(): #found word
      #print self.td_data
      self.keyword_found = True
      if self.from_lang == "de": #then get the one that is two td_data behind (using the stack)
        #print "Translation: ", self.td_data_stack[0]
        Parley.addTranslation(self.word,self.from_lang,self.to_lang,self.td_data_stack[0].strip())

    #matching on the first column (not german) and getting the translation from 2 column's on the right
    if self.in_translation_td: #found translation
      if self.from_lang != "de":
        #print "Translation: ", self.td_data
        Parley.addTranslation(self.word,self.from_lang,self.to_lang,self.td_data.strip())
      self.in_translation_td = False
      self.td_after = 0
      self.keyword_found = False

    #append td_data on the stack that keeps the last two ones
    self.td_data_stack.append(self.td_data)
    
    #make it keep only the last 2 td_data (works since we append only one item at a time)
    if len(self.td_data_stack) > 2:
      self.td_data_stack.pop(0)

    self.td_data = ""
    
    self.in_small = False

  #marks in which <td> to be able to count 2 more <td>'s and find our translation
  def start_td(self,attrs):
    if self.keyword_found == True:
        self.td_after += 1
        if self.td_after == 2:
          self.in_translation_td = True

  def handle_data(self,data):
    if self.in_small:
      return
    self.td_data += data
     
  #cleans up the given word from parentheses etc
  def clearWord(self,word):
    #word = "b[lue] socks (and) red shoes"
    p = re.compile( '(jmdn\.|etw\.)')
    word = p.sub( '', word)
    p = re.compile( '(\(.*\))')
    word = p.sub( '', word)
    p = re.compile( '(\[.*\])')
    word = p.sub( '', word)
    p = re.compile( '(\W)',re.UNICODE)
    word = p.sub( ' ', word)
    #replace double spaces produced from the previous ones
    p = re.compile( '(\s\s)')
    word = p.sub( ' ', word)
    return word.strip()





#!/usr/bin/env kf5kross
# -*- coding: utf-8 -*-
import Parley
import Kross

T = Kross.module("kdetranslation")

#FUNCTIONS

#moves selected entries to a new lesson
def moveSelectedToNewLesson():
  selected_entries = Parley.selectedEntries()
  if len(selected_entries) > 0:
    newlesson = Parley.doc.appendNewLesson(T.i18n("New Lesson"))
    for entry in selected_entries:
      newlesson.appendEntry(entry)

#marks the selected translations as known (grade 7)
def markAsKnown():
  for tr in Parley.selectedTranslations():
    tr.grade = 7

#SCRIPT MENU

#create a new action for the Scripts menu (action1)
action1 = Parley.newAction("example_action1",T.i18n("Move to new lesson"))
action1.statusTip=T.i18n("Moves the selected vocabulary to a new lesson")
Parley.connect(action1,"triggered()",moveSelectedToNewLesson)


#create a new action for the Scripts menu (action2)
action2 = Parley.newAction("example_action2",T.i18n("Mark as known (highest grade)"))
action2.statusTip=T.i18n("Sets the grade for the selected words to 7 (highest grade)")
Parley.connect(action2,"triggered()",markAsKnown)



import urllib2
import urllib
from sgmllib import SGMLParser

#"http://online-old.ectaco.com/online/diction.php3?lang=7&q=2&refid=316&pagelang=23&word=test&direction=1"

# fetches the html document for the given word and language pair
def fetchData(word,from_lang,to_lang):
  url = "http://online-old.ectaco.com/online/diction.php3"
  params = [("lang",7),("q",2),("refid",31),("pagelang",23),("word",word),("direction",1)]
  #param_word_trn = ("q",word)       #set query parameter
  #param_lang_pair = ("langpair",from_lang+"|"+to_lang)
  request_url = url + "?" + urllib.urlencode(params)
  #print request_url
  results = urllib2.urlopen(request_url)
  #results.encoding = 'windows-1250'
  return results.read().decode("cp1250")
  #return unicode(results.read(),'iso-8859-1')


#parses data and returns the parser object (that contains the translations/langpairs found)
def parseData(data):
  p = myParser()
  p.feed(data)
  p.close()
  return p

# called by Parley to translate the word
def fetchTranslation(word,from_lang,to_lang):
  data = fetchData(word,from_lang,to_lang)
  parser = parseData(data)
  return parser.words


# called by Parley to retrieve the language pairs provided by this script
# should return: [("en","fr"),("en","de")] for translation from english to french and english to german
#def getLanguagePairs():
  #data = fetchData("ignorethis","en","fr")
  #parser = parseData(data)
  #return map(split_langpair,parser.langpairs)

# function to split a language pair string into a tuple
#def split_langpair(s):
  #[f,t] = s.split("|",1)
  #return (f,t)

# ------------ HTML Parser ----------- #

class myParser(SGMLParser):
  #for every start_tagname function you add you have to make sure the tag is added to the self.tags_stack

  def reset(self):
    SGMLParser.reset(self)
    self.words = []         #translated words found in html
    self.langpairs = []     #language pairs found in html file
    self.tags_stack = []
    self.next_table_contains_translations = False
    self.inside_translation_table = False
    self.td_count = 0

  def unknown_starttag(self,tag,attrs):
        self.tags_stack.append(tag)
        #print "unknown : ", tag, " ", len(self.tags_stack)

  def start_table(self,attrs):
    if self.next_table_contains_translations == True:
      #print "next is translation table"
      self.inside_translation_table = True
      self.next_table_contains_translations = False
      self.td_count = 0
    self.tags_stack.append("table")
      
  def start_td(self,attrs):
    if self.inside_translation_table == True:
      #print "inside translation table (tdcount = ", self.td_count, ")"
      if self.td_count % 2 == 1:
        self.tags_stack.append("<!translation!>")
      else:
        self.tags_stack.append("td")
      self.td_count = self.td_count + 1
    else:
        self.tags_stack.append("td")
    
  def end_table(self):
    if self.inside_translation_table == True:
      self.inside_translation_table = False

  def handle_data(self,data):
    if len(self.tags_stack) > 0 and self.tags_stack[len(self.tags_stack)-1] == "b":
      if data.startswith("Results:"):
        #print "-", data, "-","hi"
        self.next_table_contains_translations = True

    if len(self.tags_stack) > 0 and self.tags_stack[len(self.tags_stack)-1] == "<!translation!>":
        #print "data: ", data
        Words = data.split(",")
        for w in Words:
          #print w.strip()
          self.words.append(w.strip())
  
  def unknown_endtag(self,tag):
    myParser.remove_not_closed_tags(self,tag)
    if len(self.tags_stack) > 0 and self.tags_stack[len(self.tags_stack)-1] == tag:
        #print "end_tag : ", tag, " ", len(self.tags_stack)
        self.tags_stack.pop()

  #removes all the tags from the stack that have no closed tags (don't modify)
  def remove_not_closed_tags(self,tag):
    while len(self.tags_stack) > 0 and self.tags_stack[len(self.tags_stack)-1] != tag:
      self.tags_stack.pop()



import urllib2
import urllib
from sgmllib import SGMLParser

# fetches the html document for the given word and language pair
def fetchData(word,from_lang,to_lang):
  url = "https://translate.google.com/translate_dict"
  param_word_trn = ("q",word)       #set query parameter
  param_lang_pair = ("langpair",from_lang+"|"+to_lang)
  request_url = url + "?" + urllib.urlencode([param_word_trn,param_lang_pair])
  #print request_url
  results = urllib2.urlopen(request_url)
  return results.read()

#parses data and returns the parser object (that contains the translations/langpairs found)
def parseData(data):
  p = myParser()
  p.feed(data)
  p.close()
  return p

# called by Parley to translate the word
def fetchTranslation(word,from_lang,to_lang):
  data = fetchData(word,from_lang,to_lang)
  parser = parseData(data)
  return parser.words

# called by Parley to retrieve the language pairs provided by this script
# should return: [("en","fr"),("en","de")] for translation from english to french and english to german
def getLanguagePairs():
  data = fetchData("ignorethis","en","fr")
  parser = parseData(data)
  return map(split_langpair,parser.langpairs)

# function to split a language pair string into a tuple
def split_langpair(s):
  [f,t] = s.split("|",1)
  return (f,t)

# ------------ HTML Parser ----------- #

class myParser(SGMLParser):
  #for every start_tagname function you add you have to make sure the tag is added to the self.tags_stack

  def reset(self):
    SGMLParser.reset(self)
    self.words = []         #translated words found in html
    self.langpairs = []     #language pairs found in html file
    self.tags_stack = []

  def unknown_starttag(self,tag,attrs):
        self.tags_stack.append(tag)
        #print "unknown : ", tag, " ", len(self.tags_stack)

  def start_span(self, attrs):
    #print "known : ", "span", " ", len(self.tags_stack)
    if ("class","definition") in attrs:
        self.tags_stack.append("<!translation!>") #marks tag with <!translation!> to get its data in handle_data
    else:
        self.tags_stack.append("span")

  def start_option(self, attrs):
    for name,value in attrs:
      if name == "value":
        self.langpairs.append(value)
    self.tags_stack.append("option")

  def handle_data(self,data):
    if len(self.tags_stack) > 0 and self.tags_stack[len(self.tags_stack)-1] == "<!translation!>":
        #print "data: ", data
        self.words.append(data.strip())
  
  def unknown_endtag(self,tag):
    myParser.remove_not_closed_tags(self,tag)
    if len(self.tags_stack) > 0 and self.tags_stack[len(self.tags_stack)-1] == tag:
        #print "end_tag : ", tag, " ", len(self.tags_stack)
        self.tags_stack.pop()

  #removes all the tags from the stack that have no closed tags (don't modify)
  def remove_not_closed_tags(self,tag):
    while len(self.tags_stack) > 0 and self.tags_stack[len(self.tags_stack)-1] != tag:
      self.tags_stack.pop()
